unit DOFastUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls;

type
  TDOForm = class(TForm)
    Panel1: TPanel;
    BlurbMemo: TMemo;
    Panel2: TPanel;
    LogList: TListBox;
    procedure FormCreate(Sender: TObject);
  protected
    procedure Log(LogStr: String);
    procedure WMDropFiles(var Msg: TMessage); message WM_DROPFILES;
    { Private declarations }
  public
    { Public declarations }
  end;

var
  DOForm: TDOForm;

implementation

{$R *.DFM}

uses
  ShellAPI, AIOUSB, DetectUnit;

type
  THeader = packed record
    BodyLen: LongWord;
    NameLen: Word;
    FileName: array[0..505] of AnsiChar;
  end;

var
  DeviceIndex: LongWord;

procedure TDOForm.FormCreate(Sender: TObject);
var
  PID, Status: LongWord;
  MyDetectForm: TDetectForm;
  ReadClockHz, WriteClockHz: Double;
  OutMask, TristateMask: Byte;
  DIOData: LongWord;

begin
  BlurbMemo.Lines.Text := 'This sample sends files to the DIFast sample, or equivalent. Just drag-and-drop the file(s) onto this window. The protocol is simple:'
                  + #13#10
                  + #13#10' 2 words: BodyLen, Intel DWord = length of file contents, in bytes.'
                  + #13#10' 1 word: NameLen = length of filename, in bytes.'
                  + #13#10' 253 words: Filename, in Windows ANSI, as words in Intel byte order.'
                  + #13#10' [BodyLen/2, round up to multiple of 256] words: File contents, as words in Intel byte order.'
                  + #13#10
                  + #13#10'This protocol uses the high-speed USB packet size of 512 bytes.'
  ;

  DeviceIndex := diOnly;
  Status := QueryDeviceInfo(DeviceIndex, @PID, nil, nil, nil, nil);
  if Status <> ERROR_SUCCESS then PID := 0;
  case PID of
    //$8008, //Input-only board
    $8009, //Output-only board
    //$800A, //Input-only board
    $800B,
    $800C,
    //$800D, //Input-only board
    $800E, //Output-only board
    $800F: ;
    else begin
      MyDetectForm := TDetectForm.Create(Self);
      if MyDetectForm.ShowModal = mrOK then begin
        DeviceIndex := MyDetectForm.DeviceIndex;
        MyDetectForm.Free;
      end
      else begin
        MyDetectForm.Free;
        Application.Terminate;
        Exit;
      end;
    end;
  end;

  ReadClockHz := 0; //External or "off"
  WriteClockHz := 1000000; //1MHz. Change this line to change the data rate.
  DIO_StreamSetClocks(DeviceIndex, ReadClockHz, WriteClockHz);

  DIO_StreamOpen(DeviceIndex, False);
  DIO_StreamClose(DeviceIndex);

  OutMask := $07; //A, B, and C output; D is opposite C, so it's input.
  DIOData := $FFFFFFFF;
  TristateMask := $00;
  DIO_ConfigureEx(DeviceIndex, @OutMask, @DIOData, @TristateMask);

  DragAcceptFiles(Handle, True);
  Log('Drag-and-drop files to send them.');
end;

procedure TDOForm.Log(LogStr: String);
begin
  LogList.ItemIndex := LogList.Items.Add(LogStr);
end;

function ListDroppedFiles(const Msg: TMessage): TStringList;
var
  hDrop: THandle;
  I, L, Ct: Integer;
  FN: String;

begin
  Result := TStringList.Create;

  try
    hDrop := Msg.WParam;
    Ct := DragQueryFile(hDrop, LongWord(-1), nil, 0);
    for I := 0 to Ct - 1 do begin
      L := DragQueryFile(hDrop, I, nil, 0);
      SetString(FN, nil, L + 1);
      DragQueryFile(hDrop, I, PChar(FN), L + 1);
      SetLength(FN, L);

      Result.Add(FN);
    end;

    DragFinish(hDrop);
  except
    FreeAndNil(Result);
  end;
end;

//This is the function that actually sends chunks of data(frames). This
//  sample sends files, not realtime data, so this function just sends
//  chunks until it's sent the whole file or there's an error.
function DoTransferData(TransferPackets: LongWord; Buffer: Pointer): LongWord;
var
  BytesTransferred, WordsTransferred, WordsLeft: LongWord;
  NextPt: PWord;

begin
  BytesTransferred := 0;
  WordsLeft := TransferPackets * 256;

  NextPt := PWord(Buffer);
  repeat
    Result := DIO_StreamFrame(DeviceIndex, WordsLeft, NextPt, BytesTransferred);
    if Result = ERROR_HANDLE_EOF then Result := ERROR_SUCCESS; //If it's a partial, that's fine, we'll just loop around and send the rest.
    if Result <> ERROR_SUCCESS then Break;
    //BytesTransferred will always be integral words.
    WordsTransferred := BytesTransferred div 2;
    Dec(WordsLeft, WordsTransferred);
    Inc(NextPt, WordsTransferred);
  until WordsLeft = 0;
end;

procedure TDOForm.WMDropFiles(var Msg: TMessage);
var
  FN: TStringList;
  ContentPackets, I, J, L: Integer;
  Content: String;
  Status: LongWord;
  HeaderBuf: THeader;
  MyFile: TFileStream;

begin
  FN := ListDroppedFiles(Msg);

  DIO_StreamOpen(DeviceIndex, False);
  for I := 0 to FN.Count-1 do begin
    MyFile := TFileStream.Create(FN[I], fmOpenRead or fmShareDenyNone);
    L := MyFile.Size;
    SetLength(Content, L);
    MyFile.Read(Content[1], L);
    MyFile.Free;

    FN[I] := ExtractFileName(FN[I]);
    Log('Sending file ' + FN[I] + '...');

    HeaderBuf.NameLen := Length(FN[I]);
    HeaderBuf.BodyLen := Length(Content);
    for J := 0 to HeaderBuf.NameLen-1 do
      HeaderBuf.FileName[J] := FN[I][1 + J]
    ;

    //Determine number of packets needed to send the file content.
    ContentPackets := Length(Content);
    if (ContentPackets and $1FF) <> 0 then ContentPackets := ContentPackets + $200;
    ContentPackets := ContentPackets div $200;

    L := ContentPackets * $200 - Length(Content);
    Content := Content + StringOfChar(#0, L);

    Status := DoTransferData(1, @HeaderBuf);
    if Status <> ERROR_SUCCESS then begin
      Log('Error ' + IntToStr(Status) + ' sending file header, aborting.');
    end
    else begin
      Status := DoTransferData(ContentPackets, @Content[1]);
      if Status <> ERROR_SUCCESS then begin
        Log('Error ' + IntToStr(Status) + ' sending file body, aborting.');
      end;
    end;
  end;
  DIO_StreamClose(DeviceIndex);

  Log('Done.');
end;

end.
